//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "debugdialog.h"
#include "atcommander.h"
#include "phonecontroller.h"
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qlineedit.h>

DebugDialog::DebugDialog(QWidget *parent, AtCommander *atcommander,
    PhoneController *phonecontroller) :
    DebugDialogBase(parent, "debugdialog"),
    atcommander(atcommander)
{
    this->phonecontroller = phonecontroller;

    connect(atcommander, SIGNAL(lineSent(QString)),
        this, SLOT(slotLineSent(QString)));
    connect(atcommander, SIGNAL(lineReceived(QString)),
        this, SLOT(slotLineReceived(QString)));

    connect(phonecontroller, 
        SIGNAL(statusChanged(const PhoneController::StatusInfo&, const PhoneController::StatusInfo&)),
        this, SLOT(slotPhoneStatusChanged()));
    connect(sendButton, SIGNAL(clicked()),
        this, SLOT(slotSendCommand()));
}


void DebugDialog::addLine(QString text, QString color)
{
    logTextEdit->setText(logTextEdit->text() +
        "<br><font color="+color+">" +
        text + "</font>");
    logTextEdit->scrollToBottom();
}

void DebugDialog::slotSendCommand()
{
    QString s = lineEdit->text();
    atcommander->sendAtCommand(s);
    lineEdit->setText("");
}


void DebugDialog::slotLineSent(QString s)
{
    addLine(s, "#004400");
}

void DebugDialog::slotLineReceived(QString s)
{
    addLine(s, "#440000");
}

void DebugDialog::slotPhoneStatusChanged()
{

}

DebugDialog::~DebugDialog()
{
}


#include "debugdialog.moc"
