//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "mainwindow.h"

#include <qlineedit.h>
#include <klineedit.h>
#include <qtextedit.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qcstring.h>
#include <qpushbutton.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kcmdlineargs.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <kkeydialog.h>
#include <kaction.h>
#include <ktip.h>
#include <knotifydialog.h>
#include "atcommander.h"
#include "debugdialog.h"
#include <kdebug.h>
#include <qregexp.h>
#include <libkbluetooth/rfcommsocket.h>
#include <libkbluetooth/scoserversocket.h>
#include <libkbluetooth/scosocket.h>
#include <libkbluetooth/serviceselectionwidget.h>
#include "phonenumbercompletion.h"
#include "configuredialog.h"
#include <knotifyclient.h>
#include <kfiledialog.h>
#include <qfileinfo.h>
#include <qcheckbox.h>

/// @TODO Check voice settings and display warning if they are wrong.

using namespace KBluetooth;

MainWindow::MainWindow()
{
    phonecontroller = NULL;
    debugDialog = NULL;
    bSpeechConnection = false;
    bDialOnNextConnection = false;
    peerAddress = KBluetooth::DeviceAddress::invalid;
    currentAddress = KBluetooth::DeviceAddress::invalid;
    isOutgoingRfcommConnection = false;
    KNotifyClient::startDaemon();
    dlg = new MainDialogBase(this);
    setCentralWidget(dlg);
    KTipDialog::showTip();

    dlg->dialButton->setEnabled(true);
    dlg->acceptButton->setEnabled(false);
    dlg->rejectButton->setEnabled(false);
    dlg->hangupButton->setEnabled(false);

    dialAction = new KAction(i18n("&Dial"), CTRL+Key_Return,
        this, SLOT(slotDial()), actionCollection(), "dial");
    acceptAction = new KAction(i18n("&Accept"), 0,
        this, SLOT(slotAcceptCall()), actionCollection(), "accept_call");
    rejectAction = new KAction(i18n("&Reject"), 0,
        this, SLOT(slotRejectCall()), actionCollection(), "reject_call");
    hangupAction = new KAction(i18n("&Hangup"), 0,
        this, SLOT(slotHangupCall()), actionCollection(), "hangup");
    showDebugViewAction = new KAction(i18n("&Command Viewer"), 0,
        this, SLOT(slotShowDebugView()), actionCollection(), "at_details");
    connectAction = new KAction(i18n("&Connect"), 0,
        this, SLOT(slotBtConnect()), actionCollection(), "bt_connect");
    disconnectAction = new KAction(i18n("&Disconnect"), 0,
        this, SLOT(slotBtDisconnect()), actionCollection(), "bt_disconnect");
    actionSaveAs = KStdAction::saveAs(this,
        SLOT(slotFileSaveAs()), actionCollection());

    connect(dialAction, SIGNAL(enabled(bool)),
        dlg->dialButton, SLOT(setEnabled(bool)));
    connect(acceptAction, SIGNAL(enabled(bool)),
        dlg->acceptButton, SLOT(setEnabled(bool)));
    connect(rejectAction, SIGNAL(enabled(bool)),
        dlg->rejectButton, SLOT(setEnabled(bool)));
    connect(hangupAction, SIGNAL(enabled(bool)),
        dlg->hangupButton, SLOT(setEnabled(bool)));

    setVoiceRecognitionAction = new KToggleAction(i18n("&Voice Dial"), 0,
        this, NULL, actionCollection(), "set_voice_recognition");
    soundAction = new KToggleAction(i18n("Audio Connection"), 0,
        this, NULL, actionCollection(), "sound_connection");

    KStdAction::tipOfDay(this,
        SLOT(slotForceTipOfDay()), actionCollection(), "tip_of_day");
    KStdAction::quit(kapp,
        SLOT(quit()), actionCollection());
    //KStdAction::configureToolbars(this,
    //    SLOT(optionsConfigureToolbars()), actionCollection());
    KStdAction::preferences(this,
        SLOT(optionsPreferences()), actionCollection());
    KStdAction::keyBindings(this,
        SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureNotifications(this,
        SLOT(optionsConfigureNotifications()), actionCollection());

    rejectAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "button_cancel", KIcon::Small));
    hangupAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "button_cancel", KIcon::Small));
    acceptAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "button_ok", KIcon::Small));
    dialAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "key_enter", KIcon::Small));
    connectAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "connect_established", KIcon::Small));
    disconnectAction->setIconSet(KGlobal::iconLoader()->loadIcon(
        "connect_no", KIcon::Small));

    dlg->rejectButton->setIconSet(rejectAction->iconSet());
    dlg->hangupButton->setIconSet(hangupAction->iconSet());
    dlg->acceptButton->setIconSet(acceptAction->iconSet());
    dlg->dialButton->setIconSet(dialAction->iconSet());

    actionSaveAs->setEnabled(false);

    dlg->dialLineEdit->setCompletionObject(new PhoneNumberCompletion());
    dlg->dialLineEdit->setAutoDeleteCompletionObject(true);

    //connect(dlg->dialLineEdit, SIGNAL(textChanged(const QString&)),
    //    this, SLOT(slotNumberTextChanged()));
    connect(dlg->dialButton, SIGNAL(clicked()),
        dialAction, SLOT(activate()));
    connect(dlg->acceptButton, SIGNAL(clicked()),
        acceptAction, SLOT(activate()));
    connect(dlg->rejectButton, SIGNAL(clicked()),
        rejectAction, SLOT(activate()));
    connect(dlg->hangupButton, SIGNAL(clicked()),
        hangupAction, SLOT(activate()));

    connect(dlg->voiceCheckbox, SIGNAL(toggled(bool)),
        soundAction, SLOT(setChecked(bool)));
    connect(dlg->voiceCheckbox, SIGNAL(toggled(bool)),
        soundAction, SIGNAL(toggled(bool)));
    connect(soundAction, SIGNAL(toggled(bool)),
        this, SLOT(slotToggleAudio(bool)));

    createGUI();

    statusBar()->insertItem("", 0, 4);
    statusBar()->insertFixedItem("FF:FF:FF:FF:FF:FF", 1, true);
    statusBar()->show();

    setAutoSaveSettings();
    QTimer::singleShot(0, this, SLOT(checkBluetoothSettings()));
}


void MainWindow::openConnection()
{
    DeviceAddress addr;
    int channel;
    QStringList uuids("0x111f");
    if (ServiceSelectionWidget::showSelectionDialog(this,
        uuids, addr, channel) == true) {
        RfcommSocket *socket = new RfcommSocket(this, "rfcommsocket");
        socket->connectToHost(addr, channel);
        peerAddress = addr;

        currentAddress = addr;
        outgoingChannel = channel;
        isOutgoingRfcommConnection = true;
        actionSaveAs->setEnabled(true);
        socketReInit(socket);
    }
}

void MainWindow::openConnection(KBluetooth::DeviceAddress addr,
    int channel)
{
    RfcommSocket *socket = new RfcommSocket(this, "rfcommsocket");
    socket->connectToHost(addr, channel);
    peerAddress = addr;

    currentAddress = addr;
    outgoingChannel = channel;
    isOutgoingRfcommConnection = true;
    actionSaveAs->setEnabled(true);
    socketReInit(socket);
}

void MainWindow::setNumber(QString number)
{
    dlg->dialLineEdit->setText(number);
}

void MainWindow::openConnection(int s, QString addr, QString /*devName*/)
{
    RfcommSocket *socket = new RfcommSocket(this, "rfcommsocket");
    socket->setSocket(s);
    peerAddress = DeviceAddress(addr);

    currentAddress = peerAddress;
    outgoingChannel = -1;
    isOutgoingRfcommConnection = false;
    actionSaveAs->setEnabled(false);
    socketReInit(socket);
}

void MainWindow::slotShowDebugView()
{
    if (debugDialog) debugDialog->show();
}


void MainWindow::socketReInit(RfcommSocket* socket)
{
    bSpeechConnection = false;

    if (phonecontroller) delete phonecontroller;
    phonecontroller = new PhoneController(this, "phonecontroller", socket);
    connect(phonecontroller, SIGNAL(
        statusChanged(const PhoneController::StatusInfo&, const PhoneController::StatusInfo&)),
        this, SLOT(
        slotPhoneStatusChanged(const PhoneController::StatusInfo&, const PhoneController::StatusInfo&)));
    connect(socket, SIGNAL(error(int)),
        this, SLOT(slotSocketError(int)));
    connect(socket, SIGNAL(connectionClosed()),
        this, SLOT(slotSocketClosed()));

    if (debugDialog) delete debugDialog;
    this->debugDialog = new DebugDialog(this,
        phonecontroller->getATCommander(), phonecontroller);

    phonecontroller->init();
}


void MainWindow::checkBluetoothSettings()
{
    // @TODO Try to check the settings and display warning only if neccessary
    KMessageBox::information(this, i18n("kbthandsfree needs support for SCO sockets \
in for voice transmission over bluetooth. Unfortunately, SCO is broken in many kernel \
revisions. Please refer to the online documentation of kdebluetooth for more information."),
    "KDE Bluetooth Handsfree", "refer_to_online_docs.0");
}


void MainWindow::optionsConfigureKeys()
{
    //KKeyDialog::configureKeys(actionCollection(), "kbthandsfreeui.rc");
    KKeyDialog::configure(actionCollection());
}


void MainWindow::optionsConfigureNotifications()
{
    KNotifyDialog::configure(this);
}

void MainWindow::optionsPreferences()
{
    ConfigureDialog *dlg = new ConfigureDialog(this);
    dlg->exec();
    delete dlg;
}

void MainWindow::slotPhoneStatusChanged(
    const PhoneController::StatusInfo& /*oldState*/,
    const PhoneController::StatusInfo& newState)
{
    if (newState.connected) {
        if (dialAction->isEnabled() != newState.canDial) {
            dialAction->setEnabled(newState.canDial);
        }
        if (acceptAction->isEnabled() != newState.canAccept) {
            acceptAction->setEnabled(newState.canAccept);
        }
        if (rejectAction->isEnabled() != newState.canReject) {
            rejectAction->setEnabled(newState.canReject);
        }
        if (hangupAction->isEnabled() != newState.canHangup) {
            hangupAction->setEnabled(newState.canHangup);
        }
    }
    else {
        dlg->dialButton->setEnabled(false);
        dlg->acceptButton->setEnabled(false);
        dlg->rejectButton->setEnabled(false);
        dlg->hangupButton->setEnabled(false);
    }

    QString connectionStateText;
    QString callStateText;
    if (newState.connected && newState.serviceIndicator) {
        connectionStateText = QString(newState.peerAddress);
        QStringList stateList;
        if (newState.isAlerting) {
            stateList.append(i18n("alerting %1").arg(newState.dialedNumber));
        }
        if (newState.isCalling) {
            if (newState.isIncomingCall) {
                stateList.append(i18n("calling %1").arg(newState.clipNumber));
            }
            else {
                stateList.append(i18n("calling %1").arg(newState.dialedNumber));
            }
        }
        if (newState.isDialing) {
            stateList.append(i18n("dialing %1").arg(newState.dialedNumber));
        }
        if (newState.isRinging) {
            stateList.append(QString("<font color=#BB0000>") +
                i18n("call from %1").arg(newState.clipNumber) + "</font>");
        }
        if (stateList.count() > 0) {
            callStateText = stateList.join(", ");
        }
        else {
            callStateText = "";
        }
    }
    else {
        if (newState.connected) {
            connectionStateText = i18n("No service");
        }
        else {
            connectionStateText = i18n("Not connected");
        }
        callStateText = "";
    }

    if ((newState.voiceConnection && newState.connected)!= dlg->voiceCheckbox->isChecked()) {
        dlg->voiceCheckbox->setChecked(newState.voiceConnection && newState.connected);
    }
    if (newState.connected != dlg->voiceCheckbox->isEnabled()) {
        dlg->voiceCheckbox->setEnabled(newState.connected);
    }

    statusBar()->changeItem(callStateText, 0);
    statusBar()->changeItem(connectionStateText, 1);
}


void MainWindow::slotDial()
{
    if (!phonecontroller) return;
    QString dialString = dlg->dialLineEdit->text();
    QRegExp format1(".*<([\\+-\\/0123456789]+)>");
    QRegExp format2("([\\+-\\/0123456789]+).*");
    QString number = "";
    if (format1.search(dialString) >= 0) {
        number = format1.cap(1);
    }
    else if (format2.search(dialString) >= 0) {
        number = format2.cap(1);
    }

    number = number.replace("-", "");
    number = number.replace("/", "");
    number = number.replace(" ", "");
    kdDebug() << "Dial number: " << number << endl;

    if (dlg->dialLineEdit->text().length() > 0) {
        phonecontroller->dial(number);
    }
    else {
        phonecontroller->setVoiceRecognition(true);
    }
}

void MainWindow::slotSocketError(int)
{
    QTimer::singleShot(0, this, SLOT(closeConnection()));
}

void MainWindow::slotSocketClosed()
{
    QTimer::singleShot(0, this, SLOT(closeConnection()));
}

void MainWindow::closeConnection()
{
    if (phonecontroller) {
        delete phonecontroller;
    }
    PhoneController::StatusInfo info1, info2;
    info1.connected = true;
    info2.connected = false;
    slotPhoneStatusChanged(info1, info2);
}

void MainWindow::slotFileSaveAs()
{
    QString numberString = dlg->dialLineEdit->text();
    bool saveNumber = false;
    if ((numberString.length()>0) && (KMessageBox::questionYesNo(this,
        i18n("Do you want to include the phone number into the saved shortcut?"),
        "Bluetooth Handsfree") == KMessageBox::Yes)) {
        saveNumber = true;
    }
    bool instantDial = false;
    if (saveNumber == true) {
        if (KMessageBox::questionYesNo(this,
            i18n("Do you want kbthandsfree to dial the number immediately?"),
            "Bluetooth Handsfree") == KMessageBox::Yes) {
            instantDial = true;
        }
    }
    else {
        if (KMessageBox::questionYesNo(this,
            i18n("Do you want kbthandsfree to start voice dialing immediately?"),
            "Bluetooth Handsfree") == KMessageBox::Yes) {
            instantDial = true;
        }
    }
    KFileDialog fd(QString::null, QString::null, this, "filedlg", true);
    fd.setMode(KFile::File | KFile::LocalOnly);
    fd.setSelection("handsfree shortcut");
    if (fd.exec() == QDialog::Accepted) {
        QFile outfile(fd.selectedFile());
        if (!outfile.exists() || KMessageBox::questionYesNo(this, QString(
            "%1 exists already. Do you want to overwrite it?").arg(fd.selectedFile()),
            "Bluetooth Handsfree") == KMessageBox::Yes) {
            outfile.open(IO_WriteOnly);
            KURL sdpURL;
            sdpURL.setProtocol("sdp");
            sdpURL.setHost(QString(currentAddress));
            sdpURL.setPath("/params");
            sdpURL.addQueryItem("rfcommchannel", QString::number(outgoingChannel));

            QString commandString = sdpURL.url().replace("%", "%%");
            if (saveNumber) {
                commandString = QString("--phonenumber %1 ").arg(numberString) + commandString;
            }
            if (instantDial == true) {
                commandString = QString("--dial ") + commandString;
            }

            QString desktopfile = QString(
"[Desktop Entry]\n\
Exec=kbthandsfree %2\n\
Icon=kbthandsfree\n\
Name=%1\n\
Type=Application\n")
.arg(QFileInfo(outfile.name()).fileName())
.arg(commandString);
            outfile.writeBlock(desktopfile.utf8(), desktopfile.utf8().length());
            outfile.close();
        }
    }
}

void MainWindow::slotAcceptCall()
{
    if (phonecontroller && phonecontroller->status().canAccept) {
        phonecontroller->acceptCall();
    }
}

void MainWindow::slotRejectCall()
{
    if (phonecontroller && phonecontroller->status().canReject) {
        phonecontroller->rejectCall();
    }
}

void MainWindow::slotHangupCall()
{
    if (phonecontroller && phonecontroller->status().canHangup) {
        phonecontroller->hangupCall();
    }
}

void MainWindow::slotRing()
{
    KNotifyClient::event(KApplication::kApplication()->mainWidget()->winId(),
        "IncomingCall", i18n("Incoming Call"));
}

void MainWindow::slotAskSaveRecording(QString filename)
{
    if (phonecontroller) {
        if (KMessageBox::questionYesNo(this, i18n("Save the record of this call?")) ==
            KMessageBox::Ok) {
            phonecontroller->setSaveRecording(filename, true);
        }
        else {
            phonecontroller->setSaveRecording(filename, false);
        }
    }
}

void MainWindow::slotBtConnect()
{
    slotBtDisconnect();
    openConnection();
}

void MainWindow::slotBtDisconnect()
{
    if (phonecontroller) {
        PhoneController::StatusInfo state = phonecontroller->status();
        if (state.isAlerting || state.isDialing || state.isCalling) {
            KMessageBox::information(this, i18n("<p>There is a call in progress. \
Closing this application <b>will not end the call.</b>. You will have to \
end the call using your phone</p>"));
        }
    }
    closeConnection();
}

void MainWindow::dialOnNextConnection()
{
    bDialOnNextConnection = true;
}

void MainWindow::slotInitializationFinished()
{
    if (bDialOnNextConnection) {
        bDialOnNextConnection = false;
    }
    slotDial();
}

void MainWindow::slotToggleAudio(bool bOn) {
    if (phonecontroller) {
        phonecontroller->setSpeechConnection(bOn);
    }
}

void MainWindow::slotForceTipOfDay()
{
    KTipDialog::showTip(QString::null, true);
}

MainWindow::~MainWindow()
{
    // Displays connection warning if neccessary
    slotBtDisconnect();
}

#include "mainwindow.moc"
