//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _KBTHANDSFREE_H_
#define _KBTHANDSFREE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kmainwindow.h>
#include "maindialogbase.h"
#include <libkbluetooth/deviceaddress.h>
#include <qfile.h>
#include <qguardedptr.h>
#include <phonecontroller.h>

class DebugDialog;
class SoundServer;
class KToggleAction;

namespace KBluetooth {
    class RfcommSocket;
    class ScoServerSocket;
    class ScoSocket;
}

/**
 * @short Application Main Window
 * @author Fred Schaettgen <kbluetoothd@0xF.de>
 * @version 0.1
 */
class MainWindow : public KMainWindow
{
    Q_OBJECT
public:
    MainWindow();
    virtual ~MainWindow();
    void openConnection();
    void openConnection(KBluetooth::DeviceAddress addr, int channel);
    void openConnection(int socket, QString addr, QString devName);
    void setNumber(QString number);
    void dialOnNextConnection();

private:
    void socketReInit(KBluetooth::RfcommSocket *socket);
    void addLine(QString text, QString color = "0x000000");

    MainDialogBase *dlg;
    KBluetooth::DeviceAddress peerAddress;
    //QGuardedPtr<KBluetooth::RfcommSocket> socket;
    //QGuardedPtr<AtCommander> atcommander;
    QGuardedPtr<PhoneController> phonecontroller;
    QGuardedPtr<DebugDialog> debugDialog;
    //KBluetooth::ScoServerSocket* scoServerSocket;
    //KBluetooth::ScoSocket* scoSocket;
    bool bSpeechConnection;
    KBluetooth::DeviceAddress currentAddress;
    int outgoingChannel;
    bool isOutgoingRfcommConnection;
    bool bDialOnNextConnection;

    KAction *actionSaveAs;
    /*KAction *actionAccept;
    KAction *actionReject;
    KAction *actionHangup;
    KAction *actionDial;*/

    KAction *hangupAction;
    KAction *acceptAction;
    KAction *rejectAction;
    KAction *dialAction;
    KAction *showDebugViewAction;

    KAction *connectAction;
    KAction *disconnectAction;
    KToggleAction *setVoiceRecognitionAction;
    KToggleAction *soundAction;

private slots:
    void closeConnection();
    void checkBluetoothSettings();
    void slotAskSaveRecording(QString filename);
    void slotFileSaveAs();
    void optionsPreferences();
    void optionsConfigureKeys();
    void optionsConfigureNotifications();
    void slotToggleAudio(bool bOn);
    void slotShowDebugView();
    void slotSocketError(int);
    void slotSocketClosed();
    void slotPhoneStatusChanged(
        const PhoneController::StatusInfo& oldState,
        const PhoneController::StatusInfo& newState);
    void slotDial();
    void slotAcceptCall();
    void slotRejectCall();
    void slotHangupCall();
    void slotRing();
    void slotInitializationFinished();
    void slotBtConnect();
    void slotBtDisconnect();
    void slotForceTipOfDay();
};

#endif // _KBTSERIALCHAT_H_
