//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef PHONECONTROLLER_H
#define PHONECONTROLLER_H

#include <qobject.h>
#include <qguardedptr.h>
#include <map>
#include <kaction.h>
#include <libkbluetooth/deviceaddress.h>

class AtCommander;
class SoundServer;
class VorbisEncoder;
namespace KBluetooth {
    class RfcommSocket;
}


/**
@author Fred Schaettgen
*/
class PhoneController : public QObject
{
Q_OBJECT
public:
    PhoneController(QObject *owner, const char* name,
        KBluetooth::RfcommSocket *socket);
    ~PhoneController();
    
    struct StatusInfo {
        // Connection information
        bool connected;
        KBluetooth::DeviceAddress peerAddress;
        bool voiceConnection;
        
        // Service information
        bool serviceIndicator;
        bool hasServiceIndicator;
        double signalIndicator;
        bool hasSignalIndicator;
        
        // call information
        bool isCalling;
        bool isDialing;
        bool isAlerting;
        bool isRinging;
        QString clipNumber;
        QString dialedNumber;
        bool isIncomingCall;
                
        // action flags
        bool canDial;
        bool canHangup;
        bool canReject;
        bool canAccept;    
    };
    const StatusInfo& status() const;

    bool dial(QString number);
    bool acceptCall();
    bool rejectCall();
    bool hangupCall();
    void setSpeechConnection(bool bOn);
    void setVoiceRecognition(bool bOn);
   
    
    void init();
    void setSaveRecording(QString filename, bool bSave);
    AtCommander * getATCommander() const { return atcommander; }

signals:
    //void initializationFinished();
    void statusChanged(const PhoneController::StatusInfo& oldState, 
        const PhoneController::StatusInfo& newState);
    void ring();
    void askSaveRecording(QString filename);
    //void socketError(int);
    //void socketClosed();

private:
    struct InternalStatusInfo {
        struct IndicatorInfo {
            QString name;
        };
        std::map<int, IndicatorInfo> indicators;
        std::map<QString, int> indicatorValues;
        QString clipNumber;
        QString dialedNumber;
        bool voiceRecognitionMode;
        bool isIncomingCall;
        bool voiceConnection;
        // Not all T68is support the call setup indicator
        bool callsetupIndicatorSupported;
    };
    InternalStatusInfo state;
    StatusInfo lastExternalStatus;
    void statusUpdate();
        
    bool parseCiev(QString s);
    bool parseCindRead(QString s);
    bool parseCindTest(QString s);
    bool parseRing(QString s);
    bool parseClip(QString s);
    bool parseBvra(QString s);
    void setVorbisTags();

    SoundServer *soundServer;
    VorbisEncoder *vorbisEncoder;
    QTimer *ringTimeoutTimer;

    QGuardedPtr<KBluetooth::RfcommSocket> socket;
    QGuardedPtr<AtCommander> atcommander;
    void updateStatus();
    void startStopEncoder();
    
private slots:
    void reInitT68i();
    void slotUnsolicitedResult(QString s);
    /*void hangup();
    void accept();
    void reject();
    void dial(QString number);*/
    //void slotSpeechConnectionAction(bool bOn);
    void slotSpeechConnectionStatusChanged(bool bOn);
    void slotAudioData(Q_INT16*, Q_INT16*, int);
    void slotRingTimeout();
};

#endif
