/***************************************************************************
                          ultrainquiry.h  -  description
                             -------------------
    begin                : Sat Ott 18 2003
    copyright            : (C) 2003 by Simone Gotti
    email                : motaboy@libero.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ultrainquiry.h"

#include <kglobal.h>
#include <kconfig.h>
#include <libkbluetooth/inquiry.h>
#include <libkbluetooth/namerequest.h>
#include <libkbluetooth/sdpdevice.h>
#include <libkbluetooth/sdpservice.h>
#include <libkbluetooth/rfcommsocket.h>
#include <libkbluetooth/deviceaddress.h>
#include <libkbluetooth/sdpservice.h>
#include <algorithm>

using namespace KBluetooth;
using namespace SDP;
using namespace std;


bool UltraInquiry::alreadyInserted(const KBluetooth::DeviceAddress& addr)
{

    DevInfoVector::iterator it;
    for (it = devInfos.begin(); it != devInfos.end(); ++it) {
        if(it->address() == addr)
            return true;
    }
    return false;
}

DevInfo& UltraInquiry::getDeviceInfo(const KBluetooth::DeviceAddress& addr)
{
    uint i;
    for (i = 0; i < devInfos.size(); ++i) {
        if(devInfos[i].address() == addr)
            return devInfos[i];
    }
    return devInfos[0]; // I don't know what to do but it will never happen
}

UltraInquiry::UltraInquiry(QObject *parent) :
        QObject(parent)
{
    normalInquiry = new Inquiry(NULL, this);

    readConfig();

    connect(normalInquiry, SIGNAL(neighbourFound(const KBluetooth::DeviceAddress&, int)),
            this, SLOT(slotInquiryDeviceFound(const KBluetooth::DeviceAddress&)));
    connect(normalInquiry, SIGNAL(finnished()),
            this, SLOT(slotInquiryFinnished()));

}

void UltraInquiry::inquiry()
{
    emit newDevice();
    normalInquiry->inquiry();
}

UltraInquiry::~UltraInquiry()
{
}

void UltraInquiry::readConfig()
{
    KConfig* config = KGlobal::config();
    config->setGroup("DeviceProperties");

    QStringList deviceAddrList = config->readListEntry("addresses");
    for (uint n=0; n<deviceAddrList.size(); ++n) {
        QString addrStr = deviceAddrList[n];
        DevInfo info;

        info.setAddress(KBluetooth::DeviceAddress(addrStr));

        uint servNumber = config->readNumEntry(addrStr+"_servicesnumber");
        for(uint i=0; i<servNumber; ++i) {
            ServiceInfo servInfo;
            servInfo.setStatus(ServiceInfo::Cached);
            QStringList uuidList = config->readListEntry(addrStr+"_serv"+QString::number(i)+"_classuuids");
            for(uint j=0; j < uuidList.size(); ++j) {
                QString uuidstr = uuidList[j];
                uuidstr.truncate(10); //get the first 10 chars
                KBluetooth::SDP::uuid_t uuid = KBluetooth::SDP::uuid_t(uuidstr);
                servInfo.addUUID(uuid);
            }
            uint channel = config->readUnsignedNumEntry(addrStr+"_serv"+QString::number(i)+"_rfcommchannel");
            servInfo.setRfcommChannel(channel);

            info.addService(servInfo);
        }
        info.setName(config->readEntry(addrStr+"_name"));
        info.setDiscovered(false);
        devInfos.push_back(info);
    }

    lastUsedAddress = config->readEntry("lastUsedAddress");
    if (lastUsedAddress == QString("")) {
        lastUsedAddress = QString::null;
    }
}

void UltraInquiry::writeConfig()
{
    KConfig* config = KGlobal::config();
    // First I delete the old entries;
    config->deleteGroup("DeviceProperties");

    config->setGroup("DeviceProperties");
    DevInfoVector::iterator it;
    QStringList deviceAddrList;
    for (it = devInfos.begin(); it != devInfos.end(); ++it) {
        DeviceAddress addr = it->address();
        QString addrStr = QString(addr);
        deviceAddrList.append(addrStr);

        config->writeEntry(addrStr+"_name", it->name());

        int servNumber = 0;
        DevInfo::ServiceInfoVector::iterator servIt;
        for(servIt = it->services().begin(); servIt != it->services().end(); ++servIt, servNumber++) {
            QStringList uuidList;
            KBluetooth::SDP::UUIDVec::iterator uuidIt;
            for(uuidIt = servIt->classIdList().begin(); uuidIt != servIt->classIdList().end(); ++uuidIt)
                uuidList.append(QString(*uuidIt));

            config->writeEntry(addrStr+"_serv"+QString::number(servNumber)+"_classuuids", uuidList);
            config->writeEntry(addrStr+"_serv"+QString::number(servNumber)+"_rfcommchannel", servIt->rfcommChannel());
        }
        config->writeEntry(addrStr+"_servicesnumber", servNumber);
    }
    config->writeEntry("addresses", deviceAddrList);

    // store address of last used entry
    config->writeEntry("lastUsedAddress", QString(lastUsedAddress));
}

void UltraInquiry::slotInquiryDeviceFound(const DeviceAddress& addr)
{

    if (!alreadyInserted(addr)) {
        DevInfo info;
        info.setDiscovered(true);
        info.setAddress(addr);
        info.setName(QString("<new device>"));
        devInfos.push_back(info);
    } else {
        DevInfo& info = getDeviceInfo(addr);
        info.setDiscovered(true);
    }
    emit newDevice();
}

std::vector<DevInfo> UltraInquiry::getDevices(QString classId)
{
    std::vector<DevInfo> devInfoVector;
    DevInfoVector::iterator it;
    for (it = devInfos.begin(); it != devInfos.end(); ++it) {
        if(classId != QString::null)
            if(it->haveServiceClassID(classId))
                devInfoVector.push_back(*it);
    }
    return devInfoVector;
}

void UltraInquiry::slotInquiryFinnished()
{
    // I delete all the devinfo entries that are not discovered by the inquiry process
    removeOldDevices();
    // I update the service informations
    updateDeviceInfos();
}

void UltraInquiry::removeOldDevices()
{
    DevInfoVector::iterator it = devInfos.begin();
    while(it != devInfos.end()) {
        if (it->discovered() != true)
            devInfos.erase(it);
        else
            ++it;
    }
}

void UltraInquiry::updateDeviceInfos()
{
    DevInfoVector::iterator it;
    for (it = devInfos.begin(); it != devInfos.end(); ++it) {
        if (it->discovered() != true) {
            continue;
        }
        // Clear All m_services if the list is already created
        it->clearServices();

        SDP::Device sdpDev;
        sdpDev.setAddress(it->address());

        SDP::ServiceVector services = sdpDev.services();
        SDP::ServiceVector::iterator servIt;

        for(servIt = services.begin(); servIt != services.end(); ++servIt) {
            ServiceInfo service;
            service.setClassIdList(servIt->getClassIdList());
            uint channel;
            servIt->getRfcommChannel(channel);
            service.setRfcommChannel(channel);
            service.setStatus(ServiceInfo::ServiceDiscovered);

            it->addService(service);
        }

        NameRequest nameRequest;
        it->setName(nameRequest.resolve(it->address()));
    }
    emit newDevice();
    emit finished();
    writeConfig();
}


#include "ultrainquiry.moc"
